import {
  Datasource,
  IEventParam,
  IDictionary,
  Metadata,
  Http,
} from '@zeedhi/core';
import {
  Button,
  Grid,
  Select,
  GridEditable,
  Modal,
  Form,
  ModalService,
  Switch,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class ParametersDashboardController {
  public isDisabled = true;

  public cardIsVisible = false;

  private fromExceptionOperatorsModal!: Modal;
  
  private fromOperatorsModal!: Modal;
  
  private fromProfileAccessModal!: Modal;

  get cardVisibility() {
    return !this.cardIsVisible;
  }
  
  private async deleteAndUpdate(datasource: Datasource, row: IDictionary<any>) {
    await datasource.delete(row);
    datasource.get();
  }
  
  private async enableOrDisableInitialPanel(datasource: Datasource, row: IDictionary<any>) {
    await datasource.delete(row);
    datasource.get();
  }
  
  private async enableOrDisableIndicators(datasource: Datasource, row: IDictionary<any>) {
    await datasource.delete(row);
    datasource.get();
  }
  
  public filter() {
    const grid = Metadata.getInstance<TekGrid>('parametersTekGrid');
    const [unity] = grid.getFilterInputs('CDFILIAL');
    const [panel] = grid.getFilterInputs('NMPAINEL');

    const conditions = {
      CDFILIAL: unity.value,
      NRPAINEL: panel.value,
    };
    
    grid.datasource.filter = conditions;
    grid.datasource.get();
  }

  public rowClick() {
    this.cardIsVisible = true;
  }

  public showProfileAccessCard() {
    this.cardIsVisible = false;
  }
  
  public async createProfileAccessModal() {
    this.fromProfileAccessModal = await ModalService.createFromJson('modals/addProfileAccess', true);
  }

  public destroyProfileAccessModal() {
    this.fromProfileAccessModal?.destroy();
  }

  public openProfileAccessModal() {
    this.fromProfileAccessModal.show();
  }

  public closeProfileAccessModal() {
    this.fromProfileAccessModal.hide();
  }
  
  public addProfileAccess() {
    const grid = Metadata.getInstance<Grid>('loadProfileAccessDataSourceModalHeader');
    const profileData = Metadata.getInstance<TekGrid>('parametersTekGrid').datasource.filter;
    const profileAccess = grid.getChildInstance<Select>('NMPERFILACESSO');
    const initialPanel = grid.getChildInstance<Switch>('IDPADRAODASH');
    const sendObject = {
      NRPERFILACESSO: profileAccess.value.NRPERFILACESSO,
      IDPADRAODASH: initialPanel.value,
      CDFILIAL: profileData.CDFILIAL,
      NRPAINEL: profileData.NRPAINEL,
    };
    Http.post('/addProfileAccess', sendObject);
  }
  
  public gridHasInEditinMode({ component }: IEventParam<GridEditable>) {
    this.isDisabled = !component.editing;
  }

  public async cancelEditing({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.cancelEditedRows();
    this.gridHasInEditinMode({ component: grid });
  }

  public async saveEditing({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.saveEditedRows();
    this.gridHasInEditinMode({ component: grid });
  }
  
  public clearInputsFilter() {
    const form = Metadata.getInstance<Form>('formFilterParamDash');
    form.clearForm();
  }
  
  public applyFilterDash() {
    const filter = Metadata.getInstance<Form>('formFilterParamDash');
    const grid = Metadata.getInstance<GridEditable>('parametersEditGrid');
    const conditions = {
      CDFILIAL: filter.value.CDFILIAL,
      NRPAINEL: filter.value.NRPAINEL,
    };
    
    grid.datasource.filter = conditions;
    grid.datasource.get();
  }
}
