import {
  Metadata,
  Http,
  IEventParam,
} from '@zeedhi/core';
import {
  Select,
  GridEditable,
  Button,
  Number,
  Switch,
  AlertService,
  DialogService,
} from '@zeedhi/common';
import {} from '@zeedhi/teknisa-components-common';

export class ParametersIndicatorsController {
  public savedSuccessfully() {
    AlertService.show({
      name: 'saveSucessAlert',
      text: 'Parametrização salva com sucesso.',
      color: 'success',
      timeout: 3000,
      top: true,
      right: true,
    });
  }
  
  public saveFailed() {
    AlertService.show({
      name: 'saveFailedAlert',
      text: 'Verifique os campos inválidos.',
      color: 'error',
      timeout: 5000,
      top: true,
      right: true,
    });
  }
  
  public closeConfirmationModal: any = {
    click: () => DialogService.hide(),
  };
    
  private createConfirmationDialog(cancelConfirmation) {
    return {
      name: 'buttons-dialog',
      type: 'warning',
      title: 'Cancelar Operação?',
      text: 'Todas alterações serão perdidas, deseja continuar a operação?',
      buttons: [
        {
          name: 'no', component: 'ZdButton', label: 'Não', events: this.closeConfirmationModal, outline: true,
        },
        {
          name: 'yes', component: 'ZdButton', label: 'Sim', events: cancelConfirmation,
        },
      ],
    };
  }
  
  public servicesCancelConfirmation = {
    click: async () => {
      const grid = Metadata.getInstance<GridEditable>('servicesGrid');
      await grid.cancelEditedRows();
      this.onInlineEditToggle({ component: grid });
      DialogService.hide();
    },
  };

  public unitManagerCancelConfirmation = {
    click: () => this.unitManagerBuilder(),
  };
  
  public afTrackingCancelConfirmation = {
    click: () => this.aFTrackingBuilder(),
  };
  
  public cancelChangesConfirmationServices() {
    DialogService.show(this.createConfirmationDialog(this.servicesCancelConfirmation));
  }

  public cancelChangesConfirmationUnitManager() {
    DialogService.show(this.createConfirmationDialog(this.unitManagerCancelConfirmation));
  }
  
  public cancelChangesConfirmationAfTracking() {
    DialogService.show(this.createConfirmationDialog(this.afTrackingCancelConfirmation));
  }

  public unitFilter() {
    const unitName = Metadata.getInstance<Select>('unitName');
    const grid = Metadata.getInstance<GridEditable>('servicesGrid');
    const idUnit = unitName.value;
    grid.datasource.filter = idUnit;
    const sendObject = {
      filter: [{
        name: 'CDFILIAL',
        operator: '=',
        value: idUnit,
      }],
    };
    
    Http.post('/unitServices', sendObject).then((response) => {
      const data = response.data.dataset.unitServices;
      grid.datasource.data = data;
      grid.datasource.setFilter([]);
      grid.datasource.updateData(data);
    });
  }
  
  public saveOnDB({ component }: IEventParam<GridEditable>) {
    const gridEditable = component.getEditedRows();
    Http.post('/showOnDashboard', gridEditable);
  }
  
  public isNotInlineToggleEditing = true;
  
  public onInlineEditToggle({ component }: IEventParam<GridEditable>) {
    this.isNotInlineToggleEditing = !component.editing;
  }
  
  public async saveEditingToggle({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    this.saveOnDB({ component: grid });
    await grid.saveEditedRows();
    this.onInlineEditToggle({ component: grid });
    this.savedSuccessfully();
  }
  
  public buttonStateUnitManager = true;
  
  public activateButtonUnitManager() {
    this.buttonStateUnitManager = false;
  }
  
  public unitManagerFieldsFirstTable() {
    const fieldsFirstTable = {
      productGrouping: Metadata.getInstance<Select>('productGrouping'),
      xmlSwitch: Metadata.getInstance<Switch>('xmlSwitch'),
    };
    
    return fieldsFirstTable;
  }
    
  public unitManagerFieldsSecondTable() {
    const fieldsSecondTable = {
      daysOfStashGoal: Metadata.getInstance<Number>('daysOfStashGoal'),
      daysOfStashMinimumValue: Metadata.getInstance<Number>('daysOfStashMinimumValue'),
      daysOfDelayLimit: Metadata.getInstance<Number>('daysOfDelayLimit'),
      daysOfDelayMaximumValue: Metadata.getInstance<Number>('daysOfDelayMaximumValue'),
      valueAdjustmentLimit: Metadata.getInstance<Number>('valueAdjustmentLimit'),
      valueAdjustmentMaximumValue: Metadata.getInstance<Number>('valueAdjustmentMaximumValue'),
      extraRequestsValueLimit: Metadata.getInstance<Number>('extraRequestsValueLimit'),
      extraRequestsValueMaximumValue: Metadata.getInstance<Number>('extraRequestsValueMaximumValue'),
      consumptionDeviationLimit: Metadata.getInstance<Number>('consumptionDeviationLimit'),
      consumptionDeviationMaximumValue: Metadata.getInstance<Number>('consumptionDeviationMaximumValue'),
      daysToApproveMenuLimit: Metadata.getInstance<Number>('daysToApproveMenuLimit'),
      daysToApproveMenuMaximumValue: Metadata.getInstance<Number>('daysToApproveMenuMaximumValue'),
    };
    
    return fieldsSecondTable;
  }

  public unitManagerBuilder() {
    Http.get('/unitManagerFirstTable').then((response) => {
      const unitManagerFields = this.unitManagerFieldsFirstTable();
      const data = response.data.dataset.unitManagerFirstTable;
      
      unitManagerFields.productGrouping.value = data[0].CDGRUPPROD;
      unitManagerFields.xmlSwitch.value = data[0].IDCONSXMLCOMFIL;
    });
    
    Http.get('/unitManagerSecondTable').then((response) => {
      const unitManagerFields = this.unitManagerFieldsSecondTable();
      const data = response.data.dataset.unitManagerSecondTable;
      
      unitManagerFields.daysOfStashGoal.value = data[0].VRLIMMETA;
      unitManagerFields.daysOfStashMinimumValue.value = data[0].VRLIMMINIMO;
      
      const fieldMappings = [
        'daysOfDelay', 'valueAdjustment', 'extraRequestsValue',
        'consumptionDeviation', 'daysToApproveMenu',
      ];

      fieldMappings.forEach((fieldName, index) => {
        unitManagerFields[`${fieldName}Limit`].value = data[index + 1].VRLIMMETA;
        unitManagerFields[`${fieldName}MaximumValue`].value = data[index + 1].VRLIMMINIMO;
      });
    });
    this.buttonStateUnitManager = true;
    DialogService.hide();
  }
  
  public saveUnitManager() {
    const sendObject = {
      productGrouping: Metadata.getInstance<Select>('productGrouping').value,
      xmlSwitch: Metadata.getInstance<Switch>('xmlSwitch').value,
      daysOfStashGoal: [1, Metadata.getInstance<Number>('daysOfStashGoal').value],
      daysOfStashMinimumValue: [1, Metadata.getInstance<Number>('daysOfStashMinimumValue').value],
      daysOfDelayLimit: [3, Metadata.getInstance<Number>('daysOfDelayLimit').value],
      daysOfDelayMaximumValue: [3, Metadata.getInstance<Number>('daysOfDelayMaximumValue').value],
      valueAdjustmentLimit: [4, Metadata.getInstance<Number>('valueAdjustmentLimit').value],
      valueAdjustmentMaximumValue: [4, Metadata.getInstance<Number>('valueAdjustmentMaximumValue').value],
      extraRequestsValueLimit: [5, Metadata.getInstance<Number>('extraRequestsValueLimit').value],
      extraRequestsValueMaximumValue: [5, Metadata.getInstance<Number>('extraRequestsValueMaximumValue').value],
      consumptionDeviationLimit: [19, Metadata.getInstance<Number>('consumptionDeviationLimit').value],
      consumptionDeviationMaximumValue: [19, Metadata.getInstance<Number>('consumptionDeviationMaximumValue').value],
      daysToApproveMenuLimit: [21, Metadata.getInstance<Number>('daysToApproveMenuLimit').value],
      daysToApproveMenuMaximumValue: [21, Metadata.getInstance<Number>('daysToApproveMenuMaximumValue').value],
    };
    
    const conditions = [
      {
        value: sendObject.daysOfStashMinimumValue[1],
        limit: sendObject.daysOfStashGoal[1],
      },
      {
        value: sendObject.daysOfDelayLimit[1],
        limit: sendObject.daysOfDelayMaximumValue[1],
      },
      {
        value: sendObject.valueAdjustmentMaximumValue[1],
        limit: sendObject.valueAdjustmentLimit[1],
      },
      {
        value: sendObject.extraRequestsValueMaximumValue[1],
        limit: sendObject.extraRequestsValueLimit[1],
      },
      {
        value: sendObject.consumptionDeviationMaximumValue[1],
        limit: sendObject.consumptionDeviationLimit[1],
      },
      {
        value: sendObject.daysToApproveMenuMaximumValue[1],
        limit: sendObject.daysToApproveMenuLimit[1],
      },
    ];
    
    const allConditionsPassed = conditions.every((condition) => condition.value >= condition.limit);
    
    if (allConditionsPassed) {
      this.savedSuccessfully();
      this.buttonStateUnitManager = true;
      Http.post('/saveUnitManager', sendObject);
    } else {
      this.saveFailed();
    }
  }
  
  public buttonStateAfTracking = true;
  
  public activateButtonAfTracking() {
    this.buttonStateAfTracking = false;
  }
  
  public aFTrackingFields() {
    const fields = {
      AfCriticalValue: Metadata.getInstance<Number>('AfCriticalValue'),
      AfGoal: Metadata.getInstance<Number>('AfGoal'),
      AfAnnualDeliveryGoal: Metadata.getInstance<Number>('AfAnnualDeliveryGoal'),
    };
    
    return fields;
  }
  
  public aFTrackingBuilder() {
    Http.get('/aFTrackingTable').then((response) => {
      const aFTrackingFields = this.aFTrackingFields();
      const data = response.data.dataset.aFTrackingTable;
      
      aFTrackingFields.AfCriticalValue.value = data[0].VRLIMMETA;
      aFTrackingFields.AfGoal.value = data[0].VRLIMMINIMO;
      aFTrackingFields.AfAnnualDeliveryGoal.value = data[1].VRLIMMETA;
    });
    this.buttonStateAfTracking = true;
    DialogService.hide();
  }
  
  public saveAfTracking() {
    const sendObject = {
      AfCriticalValue: [29, Metadata.getInstance<Number>('AfCriticalValue').value],
      AfGoal: [29, Metadata.getInstance<Number>('AfGoal').value],
      AfAnnualDeliveryGoal: [32, Metadata.getInstance<Number>('AfAnnualDeliveryGoal').value],
      AfEmptyField: [32, null],
    };
    const conditions = [
      {
        value: sendObject.AfGoal[1],
        limit: sendObject.AfCriticalValue[1],
      },
      {
        value: sendObject.AfAnnualDeliveryGoal[1],
        limit: sendObject.AfEmptyField[1],
      },
    ];
    
    const allConditionsPassed = conditions.every((condition) => {
      const isValueInRange = condition.value > 0 && condition.value <= 100;
      const isLimitInRange = condition.limit <= 100 && condition.limit >= 0;
      
      return condition.value >= condition.limit && isValueInRange && isLimitInRange;
    });
    
    if (allConditionsPassed) {
      this.savedSuccessfully();
      this.buttonStateAfTracking = true;
      Http.post('/saveAfTracking', sendObject);
    } else {
      this.saveFailed();
    }
  }
}
