import { Metadata, Validation } from '@zeedhi/core';
import { Number } from '@zeedhi/common';

const lessOrEqual = 'O valor deve ser maior ou igual ao campo acima';
const biggerOrEqual = 'O valor deve ser menor ou igual ao campo abaixo';

/**
 * Create a validator to check if a number is less, equal or bigger than another one.
 *
 * Once the forEach start, it will create a Validation.register function to register the validators
 * I need, with the same name as my JSON component, then I just need to call it on the field
 * that I need.
 *
 * @param metaKey: on the forEach will receive the values of the array fieldsToValidadeLessOrEqual,
 *  wich is the same name of the field on the JSON.
 * @param value: receive the value of the field where it being used.
 *
 * @return a variable with an anonymous function
 *
 * @author Regis da Cruz Freitas
 */
 
const createLessThanValidator = (metaKey: any) => {
  const VRLIMMETA = Metadata.getInstance<Number>(metaKey);
  const message = lessOrEqual;
  return (value: any) => VRLIMMETA.value <= value || message;
};

const fieldsToValidadeLessOrEqual = [
  'daysOfStashGoal',
  'daysOfDelayMaximumValue',
  'valueAdjustmentLimit',
  'extraRequestsValueLimit',
  'consumptionDeviationLimit',
  'daysToApproveMenuLimit',
  'AfCriticalValue',
];

fieldsToValidadeLessOrEqual.forEach((fieldName) => {
  Validation.register(fieldName, () => {
    const validatorFunction = createLessThanValidator(fieldName);
    return validatorFunction;
  });
});

const createBiggerThanValidator = (metaKey: any) => {
  const message = biggerOrEqual;
  return (value: any) => {
    const VRLIMMINIMO = Metadata.getInstances(metaKey)[0];
    return (VRLIMMINIMO && VRLIMMINIMO.value >= value) || message;
  };
};

const fieldsToValidadeBiggerOrEqual = [
  'daysOfStashMinimumValue',
  'daysOfDelayLimit',
  'valueAdjustmentMaximumValue',
  'extraRequestsValueMaximumValue',
  'consumptionDeviationMaximumValue',
  'daysToApproveMenuMaximumValue',
  'AfGoal',
];

fieldsToValidadeBiggerOrEqual.forEach((fieldName) => {
  Validation.register(fieldName, () => {
    const validatorFunction = createBiggerThanValidator(fieldName);
    return validatorFunction;
  });
});
